## Replication code for Figure 1 in Dolsak, Adolph, and Prakash (2020)
## "Policy Design and Public Support for Carbon Tax:
## Evidence from a 2018 U.S. National Online Survey Experiment",
## Public Administration
##
## Christopher Adolph
## 6 March 2020    www.chrisadolph.com

rm(list=ls())

## Load libraries
library(MASS)
library(simcf)
library(tile)
library(RColorBrewer)
library(Amelia)
source("insample.r")

## Select colors
brewer <- brewer.pal(9, "Set1")
red <- brewer[1]
blue <- brewer[2]
green <- brewer[3]
purple <- brewer[4]
orange <- brewer[5]
brown <- brewer[7]

## Set number of simulations
sims <- 10000

## Set post-simulation recoding of carbon tax approval
recode <- list(c(1,2, 3), c(4), c(5,6,7))

## Load data used in model -- do not delete missings
data <- read.csv("carbonData.csv")
model <-  SUPPORT  ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                     Age + White + CollegePlus + Volunteer + Children + MarriedWidow + 
                     OwnHome + Religious
carbon <- extractdata(model, data,  na.rm = FALSE)

## Impute missings using Amelia
nimp <- 100
miData <- amelia(carbon, m=nimp, ords=c(1,9))

## Make a listwise deleted version for later convenience
carbon <- extractdata(model, data,  na.rm = TRUE)


## Revise model to dummy out age by decades
model <-  SUPPORT  ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                      White + CollegePlus + Volunteer + Children + MarriedWidow + 
                      OwnHome + Religious + Age30 + Age40 + Age50 + Age60 + Age70

modelf <- SUPPORTf ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                     White + CollegePlus + Volunteer + Children + MarriedWidow + 
                     OwnHome + Religious + Age30 + Age40 + Age50 + Age60 + Age70

## Add age dummies to convenience data
carbon$Age30 <- as.numeric((carbon$Age>=30)&(carbon$Age<40))
carbon$Age40 <- as.numeric((carbon$Age>=40)&(carbon$Age<50))
carbon$Age50 <- as.numeric((carbon$Age>=50)&(carbon$Age<60))
carbon$Age60 <- as.numeric((carbon$Age>=60)&(carbon$Age<70))
carbon$Age70 <- as.numeric((carbon$Age>=70))

## Use MASS polr to do ordered probit on multiple imputation data
simbetas <- NULL
for (i in 1:nimp) {

    ## Create factor variable for polr
    miData$imputations[[i]]$SUPPORTf <- factor(miData$imputations[[i]]$SUPPORT,
                                               labels=c("Strongly Disagree",
        	                                                "Disagree",
                	                                        "Somewhat Disagree",
                        	                                "Neither Agree Nor Disagree",
                                		                 "Somewhat Agree",
                                                	        "Agree",
                                                        	"Strongly Agree"))

    ## Create age dummies in MI data
    miData$imputations[[i]]$Age30 <- as.numeric((miData$imputations[[i]]$Age>=30)&(miData$imputations[[i]]$Age<40))
    miData$imputations[[i]]$Age40 <- as.numeric((miData$imputations[[i]]$Age>=40)&(miData$imputations[[i]]$Age<50))
    miData$imputations[[i]]$Age50 <- as.numeric((miData$imputations[[i]]$Age>=50)&(miData$imputations[[i]]$Age<60))
    miData$imputations[[i]]$Age60 <- as.numeric((miData$imputations[[i]]$Age>=60)&(miData$imputations[[i]]$Age<70))
    miData$imputations[[i]]$Age70 <- as.numeric((miData$imputations[[i]]$Age>=70))
    
    ## Estimate ordered probit for this imputed dataset
    glm.resimp <- polr(modelf, data=miData$imputations[[i]], method="probit")
    
    ## Simulate parameters, add to collected simbetas
    simbetas <- rbind(simbetas,
                      mvrnorm(sims/nimp,
                              c(glm.resimp$coefficients, glm.resimp$zeta),
                              vcov(glm.resimp)
                              )
                      )
}


## Treatment list
treatments <- c("DumRevNeutr", "DumCompens", "DumMitig", "DumAdapt", "DumCntrl50")

## Create counterfactuals
xhyp <- cfMake(model, carbon, nscen=6)
scenNum <- 0

scenNum <- scenNum + 1
xhyp <- cfName(xhyp,"Revenue Neutral", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=1, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhyp <- cfName(xhyp,"Compensation", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=1, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhyp <- cfName(xhyp,"Mitigation", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=1, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhyp <- cfName(xhyp,"Adaptation", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=1, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhyp <- cfName(xhyp,"$50 Tax", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=1, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhyp <- cfName(xhyp, "Control", scen=scenNum)
xhyp <- cfChange(xhyp, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCompens", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumMitig", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhyp <- cfChange(xhyp, "DumCntrl50", x=0, xpre=0, scen=scenNum)


## Simulate SATEs collapsing presentation to three categories ("SD/D" vs "WD/N/WA" vs "SA/A")
##   with 95% CIs
oprobit.fdc <- oprobitsimfdSATE(xhyp, treatments, model, carbon, "SUPPORT", simbetas, cat=7,
                               recode=recode, ci=0.95)

##   with 90% CIs
oprobit.fdc90 <- oprobitsimfdSATE(xhyp, treatments, model, carbon, "SUPPORT", simbetas, cat=7,
                               recode=recode, ci=0.90)


#### Plot first differences

## Sort by size of effect
sortedc <- rev(order(oprobit.fdc$pe[1:5,3]))
scenNames <- row.names(xhyp$x[1:5,])

## Make main ropeladder trace
trace1c <- ropeladder(x = oprobit.fdc$pe[1:5,][sortedc,3],
                      lower = oprobit.fdc$lower[1:5,][sortedc,3],
                      upper = oprobit.fdc$upper[1:5,][sortedc,3],
                      labels = scenNames[sortedc],
                      size=0.65,
                      lex=1.75,
                      col="black",
                      lineend="square",
                      sublabels="all respondents",
                      sublabelsyoffset=0.045,
                      group=1,
                      plot=1
                      )

## Check for non-significance; overplot a white circle if ns
sigMark2b <- oprobit.fdc$pe[1:5,][sortedc,3]
is.na(sigMark2b) <- (oprobit.fdc$lower[1:5,][sortedc,3]*oprobit.fdc$upper[1:5,][sortedc,3])>0
traceSig2b <- ropeladder(x=sigMark2b,
                         col="white",
                         layer=2,
                         alpha=1,
                         group=1,
                         plot=1)

## Check for sig at 90% only; overplot a shaded circle
sigMark2b90 <- rep(NA, length(sigMark2b))
for (i in 1:length(sigMark2b)) {
    if (!is.na(sigMark2b[i])) {
        if ((oprobit.fdc90$lower[1:5,][sortedc,3][i]*oprobit.fdc90$upper[1:5,][sortedc,3][i])>0)
            sigMark2b90[i] <- oprobit.fdc$pe[1:5,][sortedc,3][i]
    }
}
traceSig2b90 <- ropeladder(x=sigMark2b90,
                           col=lighten("black", pct=0.65),
                           layer=1,
                           alpha=1,
                           group=1,
                           plot=1)


## Make a vertical line at no change
vertmark <- linesTile(x=c(0,0), y=c(0,1), plot=1)

## Set up axis marks
xat2 <- seq(from=-0.15, to=0.15, by=0.05)
xlab2 <- paste0(as.character(round(xat2,2)*100),"%")
for (i in 1:length(xat2)) if(round(xat2,2)[i]>0) xlab2[i] <- paste0("+",xlab2[i])

## Plot using tile
file <- "Figure1"
tile(trace1c, vertmark, 
     traceSig2b, traceSig2b90,
     limits=rbind(c(-0.19,0.19)),
     gridlines=list(type="xt"),
     topaxis=list(add=TRUE, at1=xat2,
                  labels1=xlab2),
     xaxis=list(at1=xat2,
                  labels1=xlab2),
     xaxistitle=list(labels1="Change in probability of support"),
     plottitle=list(labels1="Sample Average Treatment Effect"),
     width=list(plot=1.25, spacer=2),
     height=list(plottitle=3,xaxistitle=3.0,topaxistitle=3.0),
     output=list(outfile=file, width=7)
     )



