## Replication code for Figure 3 in Dolsak, Adolph, and Prakash (2020)
## "Policy Design and Public Support for Carbon Tax:
## Evidence from a 2018 U.S. National Online Survey Experiment",
## Public Administration
##
## Christopher Adolph
## 6 March 2020    www.chrisadolph.com

rm(list=ls())

## Load libraries
library(MASS)
library(simcf)
library(tile)
library(RColorBrewer)
library(Amelia)
source("insample.r")

## Select colors
brewer <- brewer.pal(9, "Set1")
red <- brewer[1]
blue <- brewer[2]
green <- brewer[3]
purple <- brewer[4]
orange <- brewer[5]
brown <- brewer[7]

## Set number of simulations
sims <- 10000

## Set post-simulation recoding of carbon tax approval
recode <- list(c(1,2, 3), c(4), c(5,6,7))


## Load data used in model -- do not delete missings
data <- read.csv("carbonData.csv")
model <-  SUPPORT  ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                     Age + White + CollegePlus + Volunteer + Children + MarriedWidow + 
                     OwnHome + Religious
carbon <- extractdata(model, data,  na.rm = FALSE)

## Impute missings using Amelia
nimp <- 100
miData <- amelia(carbon, m=nimp, ords=c(1,9))

## Make a listwise deleted version for later convenience
carbon <- extractdata(model, data,  na.rm = TRUE)

simbetas0 <- NULL
for (i in 1:nimp) {
    miData$imputations[[i]]$SUPPORTf <- factor(miData$imputations[[i]]$SUPPORT,
                                               labels=c("Strongly Disagree",
                                                        "Disagree",
                                                        "Somewhat Disagree",
                                                        "Neither Agree Nor Disagree",
                                                        "Somewhat Agree",
                                                        "Agree",
                                                        "Strongly Agree"))
}

## Revise model to dummy out age by decades
model <-  SUPPORT  ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                      White + CollegePlus + Volunteer + Children + MarriedWidow + 
                      OwnHome + Religious + Age30 + Age40 + Age50 + Age60 + Age70 +
                     DumRevNeutr:Independent + DumCompens:Independent +
                     DumMitig:Independent + DumAdapt:Independent + DumCntrl50:Independent +
                     DumRevNeutr:Republican + DumCompens:Republican +
                     DumMitig:Republican + DumAdapt:Republican + DumCntrl50:Republican

modelf <- SUPPORTf ~ DumRevNeutr + DumCompens + DumMitig + DumAdapt + DumCntrl50 +
                     Independent + Republican + CCConcern + ExtrmWthrExper + StateLegTrust + 
                     Tax2High + SpprtWthdrwParis + DrvAbove15K + IncmHshlAbvMedian + Male + 
                     White + CollegePlus + Volunteer + Children + MarriedWidow + 
                     OwnHome + Religious + Age30 + Age40 + Age50 + Age60 + Age70 +
                     DumRevNeutr:Independent + DumCompens:Independent +
                     DumMitig:Independent + DumAdapt:Independent + DumCntrl50:Independent +
                     DumRevNeutr:Republican + DumCompens:Republican +
                     DumMitig:Republican + DumAdapt:Republican + DumCntrl50:Republican

## Add age dummies to convenience data
carbon$Age30 <- as.numeric((carbon$Age>=30)&(carbon$Age<40))
carbon$Age40 <- as.numeric((carbon$Age>=40)&(carbon$Age<50))
carbon$Age50 <- as.numeric((carbon$Age>=50)&(carbon$Age<60))
carbon$Age60 <- as.numeric((carbon$Age>=60)&(carbon$Age<70))
carbon$Age70 <- as.numeric((carbon$Age>=70))

## Use MASS polr to do ordered probit on multiple imputation data
simbetas <- NULL
for (i in 1:nimp) {

   ## Create factor variable for polr
   miData$imputations[[i]]$SUPPORTf <- factor(miData$imputations[[i]]$SUPPORT,
                                               labels=c("Strongly Disagree",
                                                        "Disagree",
                                                        "Somewhat Disagree",
                                                        "Neither Agree Nor Disagree",
                                                        "Somewhat Agree",
                                                        "Agree",
                                                        "Strongly Agree"))

    ## Create age dummies in MI data
    miData$imputations[[i]]$Age30 <- as.numeric((miData$imputations[[i]]$Age>=30)&(miData$imputations[[i]]$Age<40))
    miData$imputations[[i]]$Age40 <- as.numeric((miData$imputations[[i]]$Age>=40)&(miData$imputations[[i]]$Age<50))
    miData$imputations[[i]]$Age50 <- as.numeric((miData$imputations[[i]]$Age>=50)&(miData$imputations[[i]]$Age<60))
    miData$imputations[[i]]$Age60 <- as.numeric((miData$imputations[[i]]$Age>=60)&(miData$imputations[[i]]$Age<70))
    miData$imputations[[i]]$Age70 <- as.numeric((miData$imputations[[i]]$Age>=70))
    
    ## Estimate ordered probit for this imputed dataset
    glm.resimp <- polr(modelf, data=miData$imputations[[i]], method="probit")
    
    ## Simulate parameters, add to collected simbetas
    simbetas <- rbind(simbetas,
                      mvrnorm(sims/nimp,
                              c(glm.resimp$coefficients, glm.resimp$zeta),
                              vcov(glm.resimp)
                              )
                      )
}

## Treatment list
treatments <- c("DumRevNeutr", "DumCompens", "DumMitig", "DumAdapt", "DumCntrl50")

# Make Scen C1: treatment given Democrat
xhypC1 <- cfMake(model, carbon, nscen=5)
scenNum <- 0

scenNum <- scenNum + 1
xhypC1 <- cfName(xhypC1,"Revenue Neutral", scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumRevNeutr", x=1, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCompens", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumMitig", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCntrl50", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Independent", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Republican", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhypC1 <- cfName(xhypC1,"Compensation", scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCompens", x=1, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumMitig", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCntrl50", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Independent", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Republican", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhypC1 <- cfName(xhypC1,"Mitigation", scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCompens", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumMitig", x=1, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCntrl50", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Independent", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Republican", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhypC1 <- cfName(xhypC1,"Adaptation", scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCompens", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumMitig", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumAdapt", x=1, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCntrl50", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Independent", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Republican", x=0, xpre=0, scen=scenNum)

scenNum <- scenNum + 1
xhypC1 <- cfName(xhypC1,"$50 Tax", scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumRevNeutr", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCompens", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumMitig", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumAdapt", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "DumCntrl50", x=1, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Independent", x=0, xpre=0, scen=scenNum)
xhypC1 <- cfChange(xhypC1, "Republican", x=0, xpre=0, scen=scenNum)

# Make Scen C2: treatment given Independent
xhypC2 <- xhypC1
for (i in 1:5) {
    xhypC2 <- cfChange(xhypC2, "Independent", x=1, xpre=1, scen=i)
}

# Make Scen C2: treatment given Republican
xhypC3 <- xhypC1
for (i in 1:5) {
    xhypC3 <- cfChange(xhypC3, "Republican", x=1, xpre=1, scen=i)
}

## Need a Democrat dummy in the convenience dataset
carbon$Democrat <- as.numeric((carbon$Independent==0)&(carbon$Republican==0))

## Simulate SATEs collapsing presentation to three categories ("SD/D" vs "WD/N/WA" vs "SA/A")
##   Scen C1, with 95% CIs
oprobit.fdc.C1 <- oprobitsimfdSATE(xhypC1, treatments, model, carbon[carbon$Democrat==1,],
                                   "SUPPORT", simbetas, cat=7,
                                   recode=recode, ci=0.95)

##   Scen C1, with 90% CIs
oprobit.fdc90.C1 <- oprobitsimfdSATE(xhypC1, treatments, model, carbon[carbon$Democrat==1,],
                                     "SUPPORT", simbetas, cat=7,
                                     recode=recode, ci=0.90)

##   Scen C2, with 95% CIs
oprobit.fdc.C2 <- oprobitsimfdSATE(xhypC2, treatments, model, carbon[carbon$Independent==1,],
                                   "SUPPORT", simbetas, cat=7,
                                   recode=recode, ci=0.95)

##   Scen C2, with 90% CIs
oprobit.fdc90.C2 <- oprobitsimfdSATE(xhypC2, treatments, model, carbon[carbon$Independent==1,],
                                     "SUPPORT", simbetas, cat=7,
                                     recode=recode, ci=0.90)

##   Scen C3, with 95% CIs
oprobit.fdc.C3 <- oprobitsimfdSATE(xhypC3, treatments, model, carbon[carbon$Republican==1,],
                                   "SUPPORT", simbetas, cat=7,
                                   recode=recode, ci=0.95)

##   Scen C3, with 90% CIs
oprobit.fdc90.C3 <- oprobitsimfdSATE(xhypC3, treatments, model, carbon[carbon$Republican==1,],
                                     "SUPPORT", simbetas, cat=7,
                                     recode=recode, ci=0.90)

#### Plot first differences

## Sort by size of effect -- from baseline Figure 1 model
scenNames <- c("Revenue Neutral", "Compensation", "Mitigation",  "Adaptation", "$50 Tax")
sortedc <- c(3, 1, 2, 4, 5)

## Make main ropeladder trace for scen C1: treatment given Democrat
trace1c0 <- ropeladder(x = oprobit.fdc.C1$pe[1:5,][sortedc,3],
                      lower = oprobit.fdc.C1$lower[1:5,][sortedc,3],
                      upper = oprobit.fdc.C1$upper[1:5,][sortedc,3],
                      labels = scenNames[sortedc],
                      size=0.65,
                      lex=1.75,
                      col=blue,
                      pch=15,
                      arrowsout=FALSE,
                      lineend="square",
                      entryheight=0.30, #.25
                      subentryheight=1.13,
                      sublabels="Democrats",
                      sublabelsyoffset=0.0225,
                      group=1,
                      plot=1
                      )

## Make main ropeladder trace for scen C2: treatment given Independent
trace1c <- ropeladder(x = oprobit.fdc.C2$pe[1:5,][sortedc,3],
                      lower = oprobit.fdc.C2$lower[1:5,][sortedc,3],
                      upper = oprobit.fdc.C2$upper[1:5,][sortedc,3],
                      size=0.8,
                      lex=1.75,
                      col=purple,
                      pch=18,
                      arrowsout=FALSE,
                      lineend="square",
                      sublabels="Independents",
                      sublabelsyoffset=0.0225,
                      group=2,
                      plot=1
                      )

## Make main ropeladder trace for scen C2: treatment given Republican
trace1c2 <- ropeladder(x = oprobit.fdc.C3$pe[1:5,][sortedc,3],
                      lower = oprobit.fdc.C3$lower[1:5,][sortedc,3],
                      upper = oprobit.fdc.C3$upper[1:5,][sortedc,3],
                      size=0.8,
                      lex=1.75,
                      col=red,
                      pch=18,
                      arrowsout=FALSE,
                      lineend="square",
                      sublabels="Republicans",
                      sublabelsyoffset=0.0225,
                      group=3,
                      plot=1
                      )

## Check for non-significance; overplot a white circle if ns (Scen C1)
sigMark2a <- oprobit.fdc.C1$pe[1:5,][sortedc,3]
is.na(sigMark2a) <- (oprobit.fdc.C1$lower[1:5,][sortedc,3]*oprobit.fdc.C1$upper[1:5,][sortedc,3])>0
traceSig2a <- ropeladder(x=sigMark2a,
                         col="white",
                         pch=15,
                         layer=2,
                         alpha=1,
                         group=1,
                         plot=1)

## Check for sig at 90% only; overplot a shaded circle (Scen C1)
sigMark2a90 <- rep(NA, length(sigMark2a))
for (i in 1:length(sigMark2a)) {
    if (!is.na(sigMark2a[i])) {
        if ((oprobit.fdc90.C1$lower[1:5,][sortedc,3][i]*oprobit.fdc90.C1$upper[1:5,][sortedc,3][i])>0)
            sigMark2a90[i] <- oprobit.fdc.C1$pe[1:5,][sortedc,3][i]
    }
}
traceSig2a90 <- ropeladder(x=sigMark2a90,
                           col=lighten(blue, pct=0.65),
                           pch=15,
                           layer=1,
                           alpha=1,
                           group=1,
                           plot=1)

## Check for non-significance; overplot a white circle if ns (Scen C2)
sigMark2b <- oprobit.fdc.C2$pe[1:5,][sortedc,3]
is.na(sigMark2b) <- (oprobit.fdc.C2$lower[1:5,][sortedc,3]*oprobit.fdc.C2$upper[1:5,][sortedc,3])>0
traceSig2b <- ropeladder(x=sigMark2b,
                         col="white",
                         pch=18,
                         layer=2,
                         alpha=1,
                         group=2,
                         plot=1)

## Check for sig at 90% only; overplot a shaded circle (Scen C2)
sigMark2b90 <- rep(NA, length(sigMark2b))
for (i in 1:length(sigMark2b)) {
    if (!is.na(sigMark2b[i])) {
        if ((oprobit.fdc90.C2$lower[1:5,][sortedc,3][i]*oprobit.fdc90.C2$upper[1:5,][sortedc,3][i])>0)
            sigMark2b90[i] <- oprobit.fdc.C2$pe[1:5,][sortedc,3][i]
    }
}
traceSig2b90 <- ropeladder(x=sigMark2b90,
                           col=lighten(purple, pct=0.65),
                           pch=18,
                           layer=1,
                           alpha=1,
                           group=2,
                           plot=1)

## Check for non-significance; overplot a white circle if ns (Scen C3)
sigMark2c <- oprobit.fdc.C3$pe[1:5,][sortedc,3]
is.na(sigMark2c) <- (oprobit.fdc.C3$lower[1:5,][sortedc,3]*oprobit.fdc.C3$upper[1:5,][sortedc,3])>0
traceSig2c <- ropeladder(x=sigMark2c,
                         col="white",
                         pch=18,
                         layer=2,
                         alpha=1,
                         group=3,
                         plot=1)

## Check for sig at 90% only; overplot a shaded circle (Scen C3)
sigMark2c90 <- rep(NA, length(sigMark2c))
for (i in 1:length(sigMark2c)) {
    if (!is.na(sigMark2c[i])) {
        if ((oprobit.fdc90.C3$lower[1:5,][sortedc,3][i]*oprobit.fdc90.C3$upper[1:5,][sortedc,3][i])>0)
            sigMark2c90[i] <- oprobit.fdc.C3$pe[1:5,][sortedc,3][i]
    }
}
traceSig2c90 <- ropeladder(x=sigMark2c90,
                           col=lighten(red, pct=0.65),
                           pch=18,
                           layer=1,
                           alpha=1,
                           group=3,
                           plot=1)

## Make a vertical line at no change
vertmark <- linesTile(x=c(0,0), y=c(0,1), plot=1)

## Set up axis marks
xat2 <- seq(from=-0.15, to=0.15, by=0.05)
xlab2 <- paste0(as.character(round(xat2,2)*100),"%")
for (i in 1:length(xat2)) if(round(xat2,2)[i]>0) xlab2[i] <- paste0("+",xlab2[i])

## Plot using tile
file <- "Figure3"
tile(trace1c0, trace1c, trace1c2, vertmark,  
     traceSig2a, traceSig2a90,
     traceSig2b, traceSig2b90,
     traceSig2c, traceSig2c90,
     limits=rbind(c(-0.19,0.19)),
     gridlines=list(type="xt"),
     topaxis=list(add=TRUE, at1=xat2,
                  labels1=xlab2),
     xaxis=list(at1=xat2, labels1=xlab2),
     xaxistitle=list(labels1="Change in probability of support"),
     plottitle=list(labels1="Sample Average Treatment Effect"),
     width=list(plot=1.25, spacer=2),
     height=list(plottitle=3,xaxistitle=3.0,topaxistitle=3.0),
     output=list(outfile=file,width=7)
     )
